#!/usr/bin/python3
#
# Todo 
#
#	time since last detection
#
#	sensor-crossing. SxE
#	>> patch-crossing? PxE
#
#	dont cutOff first trackEvent from plateau to end
#
#	print more info: 
#		nLines before coolOff
#		nLines after coolOff
#
# 	compat /w ecoTrack ? 1nt3gr4t3.

import sys, os

cutOff=True

LEN_TIMESTAMP=len("1602626336")
COL_TIMESTAMP=0
COL_X=6
COL_Y=COL_X+1
COL_TAG=COL_Y+1

OMIT_COL_BEGIN=4 #omit ts,ms,date,time
DELIM=","

OUT_COL_END=1
OUT_COL_LASTDETECT=3
OUT_COL_PXE=4
OUT_COL_X=5
OUT_COL_Y=OUT_COL_X+1
OUT_COL_TAG=OUT_COL_Y+1

def squash(data):

	print("squash plateaus")

	coolOff = 2
	recent = []
	filtered = []
	for line in data:
		if line[0] == '#':
			if line.startswith("#timestamp"):
				global header
				# header="#begin, end, dur, last-detect, PXE," + DELIM.join(line.split(',')[OMIT_COL_BEGIN:])
				header="#begin, end, dur, last-detect," + DELIM.join(line.split(',')[OMIT_COL_BEGIN:])
			continue

		l = line.split(DELIM)
		timestamp = int(l[COL_TIMESTAMP])
		tag = l[ COL_TAG ]
		x = l[ COL_X ]
		y = l[ COL_Y ]

		tagFound = False
		i=0
		while i < len(recent):
			
			# print(i,":",recent[i])
			samePos=(recent[i][OUT_COL_X] == x and recent[i][OUT_COL_Y] == y)
			recentBeetle=recent[i][OUT_COL_TAG] == tag
			fastBeetle = not samePos and recentBeetle
			ago = timestamp - recent[i][1]
			if ago > coolOff or fastBeetle:
				
				## add new squashed / trackevent
				filtered.append(DELIM.join(map(str,recent[i])))

				del recent[i]	# del old
				continue		# dont incr idx!

			if recentBeetle:
				
				## update
				tagFound = True
				dur = timestamp-recent[i][0]
				recent[i][1] = timestamp
				recent[i][2] = dur

			i +=1

		if not tagFound:

			## add recent
			recent.append([timestamp,timestamp,0]+l[OMIT_COL_BEGIN:])

	ncut = 0
	for r in recent:
	
		## cutOff
		if cutOff:
			r[1] = r[0]
			ncut += r[2]
			r[2] = 0

		filtered.append(DELIM.join(map(str,r)))

	if cutOff:
		print(f"cutOff {ncut} lines")

	return filtered

def add_lastDetect_and_PXE( data ):
	"add 'time since last detection' and 'patch-Cross-Event'"

	known = {} #{tag: line}

	for i,line in enumerate(data):

		l = line.split(DELIM)
		tag = l[ OUT_COL_TAG ]

		if tag not in known:
			l.insert(OUT_COL_LASTDETECT,"NA")
			# l.insert(OUT_COL_PXE,"TODO")

		else:
			last_end = int(known[tag][OUT_COL_END]) 
			this_start = int(l[OUT_COL_END])
			l.insert(OUT_COL_LASTDETECT,this_start - last_end)

		known[tag] = l

			
		data[i] = DELIM.join(map(str,l))

	return data

def write(outFile, data):

	print(f"writing to {outFile}")
	with open(outFile, "w") as f:
		f.writelines(header)
		f.writelines(data)

	cmd_compress=f"tar -zcvf {outFile}.tgz {outFile}" # ">/dev/null 2>/dev/null"
	os.system(cmd_compress)

def main(filename):

	with open(filename) as f:
		data = f.readlines()

	data = squash( data )
	data = add_lastDetect_and_PXE( data )

	write(filename + "-filtered", data)

if __name__ == "__main__" :

	if len(sys.argv) == 1:
		this = os.path.basename(__file__)
		
		sys.exit(	f'Usage:	{this} dataFile\n\n'
					f'	> Squash plateaus (start/end/dur)\n'
					f'	> CutOff tags at end (Optional)\n'
					f'	> add time-since-last-detection\n'
					f'	> add PXE - Patch Cross Event\n')

	main(sys.argv[1])
